@@ -47,11 +47,13 @@ class Instances(base.ManagerWithFind):
     """Manage :class:`Instance` resources."""
     resource_class = Instance
 
+    # TODO(SlickNik): Remove slave_of param after updating tests to replica_of
     def create(self, name, flavor_id, volume=None, databases=None, users=None,
                restorePoint=None, availability_zone=None, datastore=None,
                datastore_version=None, nics=None, configuration=None,
-               slave_of=None):
+               replica_of=None, slave_of=None):
         """Create (boot) a new instance."""
+
         body = {"instance": {
             "name": name,
             "flavorRef": flavor_id
@@ -77,8 +79,8 @@ class Instances(base.ManagerWithFind):
             body["instance"]["nics"] = nics
         if configuration:
             body["instance"]["configuration"] = configuration
-        if slave_of:
-            body["instance"]["slave_of"] = slave_of
+        if replica_of or slave_of:
+            body["instance"]["replica_of"] = replica_of or slave_of
 
         return self._create("/instances", body, "instance")
 
@@ -109,18 +111,22 @@ class Instances(base.ManagerWithFind):
         if name is not None:
             body["instance"]["name"] = name
         if detach_replica_source:
+            # TODO(glucas): Remove slave_of after updating trove
+            # (see trove.instance.service.InstanceController#edit)
             body["instance"]["slave_of"] = None
+            body["instance"]["replica_of"] = None
 
         url = "/instances/%s" % instance_id
         resp, body = self.api.client.patch(url, body=body)
         common.check_for_exceptions(resp, body, url)
 
-    def list(self, limit=None, marker=None):
+    def list(self, limit=None, marker=None, include_clustered=False):
         """Get a list of all instances.
 
         :rtype: list of :class:`Instance`.
         """
-        return self._paginated("/instances", "instances", limit, marker)
+        return self._paginated("/instances", "instances", limit, marker,
+                               {"include_clustered": include_clustered})
 
     def get(self, instance):
         """Get a specific instances.
